// DEW Example


#include <iostream.h>

#include <string.h>

#include <dew.h>


int main()
{
    // Start the engine.

    TDEWEngine engine;

    // Open a database.

    TDEWDatabase database;

    // Try to create the table "DEW_TEST.DB".

    try {

        // Declare table description variables.

        CRTblDesc descriptor;
        FLDDesc fields[3];
        IDXDesc index;

        // Zero table description variables.

        memset(&descriptor, 0, sizeof(CRTblDesc));
        memset(fields, 0, 3 * sizeof(FLDDesc));
        memset(&index, 0, sizeof(IDXDesc));

        // Make the table have three fields and one index.

        strcpy(descriptor.szTblName, "DEW_TEST.DB");
        descriptor.iFldCount = 3;
        descriptor.pfldDesc = fields;
        descriptor.iIdxCount = 1;
        descriptor.pidxDesc = &index;

        // Make first field ten characters long called "Name".

        fields[0].iFldNum = 1;
        strcpy(fields[0].szName, "Name");
        fields[0].iFldType = fldZSTRING;
        fields[0].iUnits1 = 10;

        // Make second field a date called "Date".

        fields[1].iFldNum = 2;
        strcpy(fields[1].szName, "Date");
        fields[1].iFldType = fldDATE;

        // Make third field a number called "Amount".

        fields[2].iFldNum = 3;
        strcpy(fields[2].szName, "Amount");
        fields[2].iFldType = fldFLOAT;

        // Make "Name" + "Date" the primary key.

        index.bPrimary = TRUE;
        index.iFldsInKey = 2;
        index.aiKeyFld[0] = 1;
        index.aiKeyFld[1] = 2;

        // Create the table.

        database.CreateTable(descriptor);

        cout << "Table \"DEW_TEST.DB\" creation succeeded." << endl;

    } catch(const TDEWException& exception) {

        // Report an error creating the table.

        cerr << "Table \"DEW_TEST.DB\" creation failed: " << exception << endl;

    }

    cout << endl;

    // Try to put some data in the table "DEW_TEST.DB".

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Append the data.

        cursor.InitRecord();
        record.PutField("Name", "Arnold");
        record.PutField("Date", TDate());
        record.PutField("Amount", 0);
        cursor.AppendRecord();

        cursor.InitRecord();
        record.PutField(1, "Bruce");
        record.PutField(2, TDate(31, 10, 1994));
        record.PutField(3, 5.00);
        cursor.AppendRecord();

        cursor.InitRecord();
        record.PutField(1, "Charles");
        record.PutField(2, TDate(31, 10, 1994));
        record.PutBlank(3);
        cursor.AppendRecord();

        cursor.InitRecord();
        record.PutField(1, "Charles");
        record.PutField(2, TDate(1, 11, 1994));
        record.PutField(3, -2.50);
        cursor.AppendRecord();

        cout << "Records successfully added." << endl;

    } catch(const TDEWException& exception) {

        // Report an error appending the data.

        cerr << "Records could not be added: " << exception << endl;

    }

    cout << endl;

    // Try to put some data in the table "DEW_TEST.DB" that violates its primary key.

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Append some data that has the same key as data previously added.

        cursor.InitRecord();
        record.PutField("Name", "Arnold");
        record.PutField("Date", TDate());
        record.PutField("Amount", 10);
        try {
            cursor.AppendRecord();
            cerr << "Key violation not detected." << endl;
        } catch (const TDEWIntegrityException& exception) {
            if (exception.Result() == DBIERR_KEYVIOL)
                cout << "Key violation successfully detected." << endl;
            else
                throw;
        }

    } catch(const TDEWException& exception) {

        // Report a failure to catch the key violation.

        cerr << "Key violation not detected." << endl;

    }

    cout << endl;

    // Try to extract the data from the table "DEW_TEST.DB".

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Print the attributes (field) names.

        cout << "Reading (Name, Date, Amount) . . . " << endl;

        // Print all of the tuples (records).

        for (cursor.SetToBegin(); cursor.Advance(); ) {
            string name;
            TDate date;
            float amount;
            record.GetField("Name", name);
            record.GetField("Date", date);
            record.GetField("Amount", amount);
            date.SetPrintOption(TDate::Numbers);
            cout << "\t(\"" << name << "\", " << date << ", " << amount << ")" << endl;
        }

        cout << "Data successfully read." << endl;

    } catch(const TDEWException& exception) {

        // Report an error reading the data.

        cerr << "Data could not be read: " << exception << endl;

    }

    cout << endl;

   // Try to search for a key value in "DEW_TEST.DB".

    try {

        // Create a cursor for the table and find its record buffer.

        TDEWCursor cursor(database, "DEW_TEST.DB");
        TDEWRecord& record = cursor.Record();

        // Create a record buffer for the key.

        TDEWRecord key(cursor);

        // Search for the key.

        key.PutField("Name", "Bruce");
        cout << "Searching for (\"Bruce\", *, *) . . ." << endl;
        cursor.GetRecordForKey(key, 1);
        string name;
        TDate date;
        float amount;
        record.GetField("Name", name);
        record.GetField("Date", date);
        record.GetField("Amount", amount);
        date.SetPrintOption(TDate::Numbers);
        cout << "\t(\"" << name << "\", " << date << ", " << amount << ")" << endl;

        cout << "Search successfully completed." << endl;

    } catch(const TDEWException& exception) {

        // Report a failure of the search.

        cerr << endl << "Key could not be found: " << exception << endl;

    }

    cout << endl;

    // Try to delete the table "DEW_TEST.DB".

    try {

        // Delete the table.

        database.DeleteTable("DEW_TEST.DB");

        cout << "Table successfully deleted." << endl;

    } catch(const TDEWException& exception) {

        // Report an error deleting the table.

        cerr << "Table could not be deleted: " << exception << endl;

    }

    return 0;
}
