//
//  Database Engine Wrapper
//  Version 1.0
//  RECORD.CPP
//  Copyright  1994
//  B. W. Bush
//  All rights reserved
//


// Definitions.

#include <dew.h>        // database engine wrapper


// Constructor.

TDEWRecord::TDEWRecord(TDEWCursor& cursor)
  : fCursor(cursor),
    fBuffer(0)
{
    fBuffer = new BYTE[fCursor.fCursorProps.iRecBufSize];
}


// Destructor.

TDEWRecord::~TDEWRecord()
{
    delete [] fBuffer;
}


// Get the size of a blob.

UINT32 TDEWRecord::BlobSize(const string& fieldName) const
{
    return BlobSize(FieldNumber(fieldName));
}

UINT32 TDEWRecord::BlobSize(const UINT16 iField) const
{
    CheckFieldNumber(iField);

    UINT32 iSize;

    TDEWException::CheckResult(DbiGetBlobSize(fCursor.fHCursor, fBuffer, iField, &iSize));

    return iSize;
}


// Answer the record's fCursor.

TDEWCursor& TDEWRecord::Cursor()
{
    return fCursor;
}

const TDEWCursor& TDEWRecord::Cursor() const
{
    return fCursor;
}


// Find the field name.

string TDEWRecord::FieldName(const UINT16 iField) const
{
    CheckFieldNumber(iField);

    return string(fCursor.fFieldDesc[iField - 1].szName);
}


// Find the field number.

UINT16 TDEWRecord::FieldNumber(const string& fieldName) const
{
    for (UINT16 i = 0; i < fCursor.fCursorProps.iFields; ++i)
        if (fieldName == fCursor.fFieldDesc[i].szName)
            return i + 1;

    TDEWException::CheckResult(DBIERR_INVALIDFIELDNAME);

    return -1;
}


// Check for a blank.

BOOL TDEWRecord::IsBlank(const string& fieldName) const
{
    return IsBlank(FieldNumber(fieldName));
}

BOOL TDEWRecord::IsBlank(const UINT16 iField) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;

    TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, NULL, &bBlank));

    return bBlank;
}


// Get a field as a string.

BOOL TDEWRecord::GetField(const string& fieldName, string& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, string& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];
    char temp[256];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            temp[0] = 0;
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldZSTRING:
                case fldBYTES:
                    strcpy(temp, dest);
                    break;
                case fldDATE:
                    {
                        UINT16 mo, da; INT16 ye;
                        TDEWException::CheckResult(DbiDateDecode(*(DATE*) dest, &mo, &da, &ye));
                        sprintf(temp, "%02u/%02u/%i", mo, da, ye);
                    }
                    break;
                case fldBOOL:
                    strcpy(temp, *(BOOL*) dest == 0 ? "False" : "True");
                    break;
                case fldINT16:
                    sprintf(temp, "%i", *(INT16*) dest);
                    break;
                case fldINT32:
                    sprintf(temp, "%li", *(INT32*) dest);
                    break;
                case fldFLOAT:
                    sprintf(temp, (fCursor.fFieldDesc[iField - 1].iSubType == fldstMONEY ? "$%lg" : "%lg"), *(FLOAT*) dest);
                    break;
                case fldBCD:
                    {
                        FLOAT x;
                        TDEWException::CheckResult(DbiBcdToFloat((pFMTBcd) dest, &x));
                        sprintf(temp, "%lg", x);
                    }
                    break;
                case fldTIME:
                    {
                        UINT16 ho, mi, se;
                        TDEWException::CheckResult(DbiTimeDecode(*(TIME*) dest, &ho, &mi, &se));
                        sprintf(temp, "%02u:%02u:%02i", ho, mi, se / 1000);
                    }
                    break;
                case fldTIMESTAMP:
                    {
                        DATE date;
                        TIME time;
                        UINT16 da, mo, ho, mi, se; INT16 ye;
                        TDEWException::CheckResult(DbiTimeStampDecode(*(TIMESTAMP*) dest, &date, &time));
                        TDEWException::CheckResult(DbiDateDecode(date, &da, &mo, &ye));
                        TDEWException::CheckResult(DbiTimeDecode(time, &ho, &mi, &se));
                        sprintf(temp, "%02u/%02u/%i %02u:%02u:%02d", da, mo, ye, ho, mi, se / 1000);
                    }
                    break;
                case fldUINT16:
                    sprintf(temp, "%u", *(UINT16*) dest);
                    break;
                case fldUINT32:
                    sprintf(temp, "%lu", *(UINT32*) dest);
                    break;
                case fldFLOATIEEE:
                    sprintf(temp, "%Lg", *(long double*) dest);
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    answer = string(temp);

    delete [] dest;

    return bBlank;
}


// Get a field as an integer.

BOOL TDEWRecord::GetField(const string& fieldName, INT16& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, INT16& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = 0;
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldBOOL:
                    answer = *(BOOL*) dest;
                    break;
                case fldINT16:
                    answer = *(INT16*) dest;
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}

BOOL TDEWRecord::GetField(const string& fieldName, INT32& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, INT32& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = 0;
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldINT16:
                    answer = *(INT16*) dest;
                    break;
                case fldINT32:
                    answer = *(INT32*) dest;
                    break;
                case fldUINT16:
                    answer = *(UINT16*) dest;
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
        }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}

BOOL TDEWRecord::GetField(const string& fieldName, UINT16& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, UINT16& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = 0;
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldUINT16:
                    answer = *(UINT16*) dest;
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}

BOOL TDEWRecord::GetField(const string& fieldName, UINT32& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, UINT32& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = 0;
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldUINT16:
                    answer = *(UINT16*) dest;
                    break;
                case fldUINT32:
                    answer = *(UINT32*) dest;
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}


// Get field as a real.

BOOL TDEWRecord::GetField(const string& fieldName, float& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, float& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = 0;
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldINT16:
                    answer = *(INT16*) dest;
                    break;
                case fldINT32:
                    answer = *(INT32*) dest;
                    break;
                case fldFLOAT:
                    answer = (float) *(FLOAT*) dest;
                    break;
                case fldBCD:
                    {
                        FLOAT x;
                        TDEWException::CheckResult(DbiBcdToFloat((pFMTBcd) dest, &x));
                        answer = (float) x;
                    }
                    break;
                case fldUINT16:
                    answer = *(UINT16*) dest;
                    break;
                case fldUINT32:
                    answer = *(UINT32*) dest;
                    break;
                case fldFLOATIEEE:
                    answer = (float) *(long double*) dest;
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}

BOOL TDEWRecord::GetField(const string& fieldName, double& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, double& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = 0;
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldINT16:
                    answer = *(INT16*) dest;
                    break;
                case fldINT32:
                    answer = *(INT32*) dest;
                    break;
                case fldFLOAT:
                    answer = *(FLOAT*) dest;
                    break;
                case fldBCD:
                    {
                        FLOAT x;
                        TDEWException::CheckResult(DbiBcdToFloat((pFMTBcd) dest, &x));
                        answer = x;
                    }
                    break;
                case fldUINT16:
                    answer = *(UINT16*) dest;
                    break;
                case fldUINT32:
                    answer = *(UINT32*) dest;
                    break;
                case fldFLOATIEEE:
                    answer = (double) *(long double*) dest;
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}

BOOL TDEWRecord::GetField(const string& fieldName, long double& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, long double& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = 0;
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldINT16:
                    answer = *(INT16*) dest;
                    break;
                case fldINT32:
                    answer = *(INT32*) dest;
                    break;
                case fldFLOAT:
                    answer = *(FLOAT*) dest;
                    break;
                case fldBCD:
                    {
                        FLOAT x;
                        TDEWException::CheckResult(DbiBcdToFloat((pFMTBcd) dest, &x));
                        answer = x;
                    }
                    break;
                case fldUINT16:
                    answer = *(UINT16*) dest;
                    break;
                case fldUINT32:
                    answer = *(UINT32*) dest;
                    break;
                case fldFLOATIEEE:
                    answer = *(long double*) dest;
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}


// Get a field as a date.

BOOL TDEWRecord::GetField(const string& fieldName, TDate& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, TDate& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = TDate();
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldDATE:
                    {
                        UINT16 mo, da; INT16 ye;
                        TDEWException::CheckResult(DbiDateDecode(*(DATE*) dest, &mo, &da, &ye));
                        answer = TDate(da, mo, ye);
                    }
                    break;
                case fldTIMESTAMP:
                    {
                        DATE date;
                        TIME time;
                        TDEWException::CheckResult(DbiTimeStampDecode(*(TIMESTAMP*) dest, &date, &time));
                        UINT16 da, mo; INT16 ye;
                        TDEWException::CheckResult(DbiDateDecode(date, &da, &mo, &ye));
                        answer = TDate(da, mo, ye);
                    }
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}


// Get a field as a time.

BOOL TDEWRecord::GetField(const string& fieldName, TTime& answer) const
{
    return GetField(FieldNumber(fieldName), answer);
}

BOOL TDEWRecord::GetField(const UINT16 iField, TTime& answer) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank)
            answer = TTime();
        else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldTIME:
                    {
                        UINT16 ho, mi, se;
                        TDEWException::CheckResult(DbiTimeDecode(*(TIME*) dest, &ho, &mi, &se));
                        answer = TTime(ho, mi, se / 1000);
                    }
                    break;
                case fldTIMESTAMP:
                    {
                        DATE date;
                        TIME time;
                        TDEWException::CheckResult(DbiTimeStampDecode(*(TIMESTAMP*) dest, &date, &time));
                        UINT16 ho, mi, se;
                        TDEWException::CheckResult(DbiTimeDecode(time, &ho, &mi, &se));
                        answer = TTime(ho, mi, se / 1000);
                    }
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}


// Get a field as a time stamp.

BOOL TDEWRecord::GetField(const string& fieldName, TDate& answer1, TTime& answer2) const
{
    return GetField(FieldNumber(fieldName), answer1, answer2);
}

BOOL TDEWRecord::GetField(const UINT16 iField, TDate& answer1, TTime& answer2) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;
    char* dest = new char[fCursor.fFieldDesc[iField - 1].iLen];

    try {

        TDEWException::CheckResult(DbiGetField(fCursor.fHCursor, iField, fBuffer, (pBYTE) dest, &bBlank));

        if (bBlank) {
            answer1 = TDate();
            answer2 = TTime();
        } else
            switch (fCursor.fFieldDesc[iField - 1].iFldType) {
                case fldTIMESTAMP:
                    {
                        DATE date;
                        TIME time;
                        TDEWException::CheckResult(DbiTimeStampDecode(*(TIMESTAMP*) dest, &date, &time));
                        UINT16 da, mo; INT16 ye;
                        TDEWException::CheckResult(DbiDateDecode(date, &da, &mo, &ye));
                        answer1 = TDate(da, mo, ye);
                        UINT16 ho, mi, se;
                        TDEWException::CheckResult(DbiTimeDecode(time, &ho, &mi, &se));
                        answer2 = TTime(ho, mi, se / 1000);
                    }
                    break;
                default:
                    TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
            }

    } catch(...) {

        delete [] dest;

        throw;

    }

    delete [] dest;

    return bBlank;
}


// Put a blank into a field.

void TDEWRecord::PutBlank(const string& fieldName)
{
    PutBlank(FieldNumber(fieldName));
}

void TDEWRecord::PutBlank(const UINT16 iField)
{
    CheckFieldNumber(iField);

    TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, NULL));
}


// Put a field as a string.

void TDEWRecord::PutField(const string& fieldName, const string& answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const string& answer)
{
    CheckFieldNumber(iField);
    if (answer.length() > 255)
        TDEWException::CheckResult(DBIERR_INVALIDPARAM);
        
    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldZSTRING:
        case fldBYTES:
            {
                char sour[256];
                strcpy(sour, answer.c_str());
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}


// Put a field as an integer.

void TDEWRecord::PutField(const string& fieldName, const INT16 answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const INT16 answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldINT16:
            {
                INT16 sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldINT32:
            {
                INT32 sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOAT:
            {
                FLOAT sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldBCD:
            {
                FLOAT x = answer;
                FMTBcd sour;
                TDEWException::CheckResult(DbiBcdFromFloat(&x, fCursor.fFieldDesc->iUnits1, fCursor.fFieldDesc->iUnits2, &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOATIEEE:
            {
                long double sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}

void TDEWRecord::PutField(const string& fieldName, const INT32 answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const INT32 answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldINT32:
            {
                INT32 sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOAT:
            {
                FLOAT sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldBCD:
            {
                FLOAT x = answer;
                FMTBcd sour;
                TDEWException::CheckResult(DbiBcdFromFloat(&x, fCursor.fFieldDesc->iUnits1, fCursor.fFieldDesc->iUnits2, &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOATIEEE:
            {
                long double sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}

void TDEWRecord::PutField(const string& fieldName, const UINT16 answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const UINT16 answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldUINT16:
            {
                UINT16 sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldUINT32:
            {
                UINT32 sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOAT:
            {
                FLOAT sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldBCD:
            {
                FLOAT x = answer;
                FMTBcd sour;
                TDEWException::CheckResult(DbiBcdFromFloat(&x, fCursor.fFieldDesc->iUnits1, fCursor.fFieldDesc->iUnits2, &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOATIEEE:
            {
                long double sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}

void TDEWRecord::PutField(const string& fieldName, const UINT32 answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const UINT32 answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldUINT32:
            {
                UINT32 sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOAT:
            {
                FLOAT sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldBCD:
            {
                FLOAT x = answer;
                FMTBcd sour;
                TDEWException::CheckResult(DbiBcdFromFloat(&x, fCursor.fFieldDesc->iUnits1, fCursor.fFieldDesc->iUnits2, &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOATIEEE:
            {
                long double sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}


// Put a field as a real.

void TDEWRecord::PutField(const string& fieldName, const float answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const float answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldFLOAT:
            {
                FLOAT sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldBCD:
            {
                FLOAT x = answer;
                FMTBcd sour;
                TDEWException::CheckResult(DbiBcdFromFloat(&x, fCursor.fFieldDesc->iUnits1, fCursor.fFieldDesc->iUnits2, &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOATIEEE:
            {
                long double sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}

void TDEWRecord::PutField(const string& fieldName, const double answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const double answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldFLOAT:
            {
                FLOAT sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldBCD:
            {
                FLOAT x = answer;
                FMTBcd sour;
                TDEWException::CheckResult(DbiBcdFromFloat(&x, fCursor.fFieldDesc->iUnits1, fCursor.fFieldDesc->iUnits2, &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOATIEEE:
            {
                long double sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}

void TDEWRecord::PutField(const string& fieldName, const long double answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const long double answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldFLOAT:
            {
                FLOAT sour = (FLOAT) answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldBCD:
            {
                FLOAT x = (FLOAT) answer;
                FMTBcd sour;
                TDEWException::CheckResult(DbiBcdFromFloat(&x, fCursor.fFieldDesc->iUnits1, fCursor.fFieldDesc->iUnits2, &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        case fldFLOATIEEE:
            {
                long double sour = answer;
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}


// Put a field as a date.

void TDEWRecord::PutField(const string& fieldName, const TDate& answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const TDate& answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldDATE:
            {
                DATE sour;
                TDEWException::CheckResult((UINT16) DbiDateEncode(answer.Month(), (UINT16) answer.DayOfMonth(), (INT16) answer.Year(), &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}


// Put a field as a time.

void TDEWRecord::PutField(const string& fieldName, const TTime& answer)
{
    PutField(FieldNumber(fieldName), answer);
}

void TDEWRecord::PutField(const UINT16 iField, const TTime& answer)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldTIME:
            {
                TIME sour;
                TDEWException::CheckResult((UINT16) DbiTimeEncode(answer.Hour(), (UINT16) answer.Minute(), (UINT16) answer.Second(), &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}


// Put a field as a time stamp.

void TDEWRecord::PutField(const string& fieldName, const TDate& answer1, const TTime& answer2)
{
    PutField(FieldNumber(fieldName), answer1, answer2);
}

void TDEWRecord::PutField(const UINT16 iField, const TDate& answer1, const TTime& answer2)
{
    CheckFieldNumber(iField);

    switch (fCursor.fFieldDesc[iField - 1].iFldType) {
        case fldTIMESTAMP:
            {
                DATE date;
                TDEWException::CheckResult((UINT16) DbiDateEncode(answer1.Month(), (UINT16) answer1.DayOfMonth(), (INT16) answer1.Year(), &date));
                TIME time;
                TDEWException::CheckResult((UINT16) DbiTimeEncode(answer2.Hour(), (UINT16) answer2.Minute(), (UINT16) answer2.Second(), &time));
                TIMESTAMP sour;
                TDEWException::CheckResult(DbiTimeStampEncode(date, time, &sour));
                TDEWException::CheckResult(DbiPutField(fCursor.fHCursor, iField, fBuffer, (pBYTE) &sour));
            }
            break;
        default:
            TDEWException::CheckResult(DBIERR_NOTSUPPORTED);
    }
}


// Verify the field value.

BOOL TDEWRecord::VerifyField(const string& fieldName) const
{
    return VerifyField(FieldNumber(fieldName));
}

BOOL TDEWRecord::VerifyField(const UINT16 iField) const
{
    CheckFieldNumber(iField);

    BOOL bBlank;

    TDEWException::CheckResult(DbiVerifyField(fCursor.fHCursor, iField, fBuffer, &bBlank));

    return bBlank;
}
